#LTPratio.py
#This calculates area under the curve for two or more stimulation conditions and two or more molecule species.  
#This is similar to LTPratioMultiSpine but does not assume the .avg file consists of a list of spine averages.  
#Instead, user must explicity specify each column to process.
#If more than 2 molecules, can specifiy which molecules to use in the ratio using the ratiomols variable
#The input file is output from NRDPostAB to calculate region averages of a single molecule species.

import os
from numpy import *
from pylab import *
import sys

#give basic information such as directory, root name of the files, and list of molecules to be processed:

#specify complete name of file, excluding molecule name, can process multiple simulations
fname=[sys.argv[1]+'-']
#fname=['12mar20-Dgl600-']

condition=['Theta','20hz']

#This list of molecules can be of arbitrary length. However, the ratio stuff might not work if not two molecules 
mollist=['Pkct', '2ag']   
ratiomols=[0,1]
#which columns of data do you want to process, e.g. spinehead and mean
#the last value will be replaced by 2nd to last column below.
cols=[11,17]
basaltime=20000
stimtime=20000
endtime=250000

#*************Use assignments above to sum corresponding columns in files *******************
#this loop assigns the filenames
for filenm in range(len(fname)):
    auc=zeros((len(condition),(len(mollist)+1)*len(cols)))
    for cond in range(len(condition)):
        for mol in range(len(mollist)):
            fullfilename=condition[cond]+fname[filenm]+mollist[mol]+'_avg.txt'
                
#this loop reads in the data arrays and adds them together
            data=loadtxt(fullfilename,skiprows=1)
            print fullfilename,data.shape
                
#make sure the second to last column is identified, and extract the time column
            cols[len(cols)-1]=data.shape[1]-2
            time=data[:,0]
            
#calculate area under the curve from the beginning of stimulation through when the response returns to basal.
            dt=time[1]-time[0]
            basalend=basaltime/dt
            stimsample=stimtime/dt
            endsample=endtime/dt
            for col in range(len(cols)):
                auc[cond,col*(len(mollist)+1)+mol]=(endtime-stimtime)*(data[stimsample:endsample,cols[col]].mean()-data[0:basalend,cols[col]].mean())/1000/1000
                print condition[cond], "col=", cols[col], "mol=", mollist[mol], "auccol=", col*(len(mollist)+1)+mol
        #AUC now has been calculated for each column for all molecules
        #Now, calculate ratio, assuming only 2 molecules
        for col in range(len(cols)):
            auc[cond,col*(len(mollist)+1)+len(mollist)]=auc[cond,col*(len(mollist)+1)+ratiomols[0]]/auc[cond,col*(len(mollist)+1)+ratiomols[1]]

#output the data
    outfname=fname[filenm]+'auc.txt'
    header='cond '
    for col in cols:
        for mol in mollist:
            header=header+'  '+mol+str(col)
        header=header+' ratio'+str(col)
    header=header+'\n'
    print outfname, "header:", header
    savetxt(outfname, auc, fmt='%.3f', delimiter=' ')
    f=open(outfname,'r')
    temp=f.readlines()
    f.close()
    f=open(outfname,'w')
    for cond in range(len(condition)):
        temp[cond]=condition[cond]+' '+temp[cond]
    temp.insert(0,header)
    f.writelines(temp)
    f.close()
    
       #End of for cond loop
    

