#This calculates area under the curve for two or more stimulation conditions and two or more molecule species.
#The input file is output from NRDPostAB to calculate region averages of a single molecule species.
#It also calculates the ratio of the first molecule to the second molecule in the list for each region
import os
from numpy import *
from pylab import *
import sys

#give basic information such as directory, root name of the files, and list of molecules to be processed:

#specify complete name of file, excluding molecule name, can process multiple simulations
#Presently it loops over prefix and condition, but that can be changed easily
#This list of stimulation conditions  can be of arbitrary length
condition=['Theta','20hz']
prefix=['']

#This list of molecules can be of arbitrary length.  
mollist=['Pkct', '2ag']   

#For processing PKCt to 2AG ratio in spines, specify the number of spines in the morphology
firstspinecol=2
numspines=13
spinecol=range(numspines)
print spinecol
basaltime=20000
stimtime=20000
endtime=250000

#*************Use assignments above to sum corresponding columns in files *******************
#this loop assigns the filenames
for filenm in range(len(prefix)):
    auc=zeros((numspines,2*(len(mollist)+1)))
    for cond in range(len(condition)):
        for mol in range(len(mollist)):
            fullfilename=condition[cond]+sys.argv[1]+'-'+mollist[mol]+'_avg.txt'
                
#this loop reads in the data arrays
            data=loadtxt(fullfilename,skiprows=1)
            print fullfilename,data.shape
                
#make sure the second to last column is identified, and extract the time column
            meancol=data.shape[1]-2
            time=data[:,0]
            
#calculate area under the curve from the beginning of stimulation through when the response returns to basal.
            dt=time[1]-time[0]
            basalend=basaltime/dt
            stimsample=stimtime/dt
            endsample=endtime/dt
            for spine in spinecol:
                auc[spine,cond*(len(mollist)+1)+mol]=(endtime-stimtime)*(data[stimsample:endsample,spine+firstspinecol].mean()
                                                   -data[0:basalend,spine+firstspinecol].mean())/1000/1000
                print "cond,mol,spine,datacol,auccol:", condition[cond], mol, spine, spine+firstspinecol, cond*(len(mollist)+1)+mol, auc[spine,cond*(len(mollist)+1)+mol]
       #AUC now has been calculated for each spine of both molecules
        #Now, calculate ratio for each spine
        for spine in spinecol:
            auc[spine,cond*(len(mollist)+1)+2]=auc[spine,cond*(len(mollist)+1)+0]/auc[spine,cond*(len(mollist)+1)+1]

#output the data
    outfname=sys.argv[1]+'-auc.txt'
    header='spine '
    for cond in range(len(condition)):
        for mol in mollist:
            header=header+' '+condition[cond]+mol
        header=header+' ratio'
    header=header+'\n'
    print outfname, "header:", header
    outdata=column_stack((spinecol,auc))

    f=open(outfname,'w')
    f.write(header)
    savetxt(f, outdata, fmt='%.3f', delimiter=' ')
    f.close()

    #End of for cond loop
    

