#Python version of NRDpost - use for volumes with too many voxels for NRDpost
#Assumes each molecule in a separate file, but multiple molecules can be specified in mollist
#Also assumes that molecule outputs are integers
#Can process multiple stimulation conditions, but all files must use the same morphology, and thus 
#have the same meshfile.  Headername is the name of the file which first line has all the column labels.
#It will provide region averages (each spine, dendrite submembrane, cytosol) and if spatialaverage=yes,
#will calculate an average of 10 segments along the dendrite, 
#or whatever structure name is specified in dend variable
import os
from numpy import *
from pylab import *
from string import *

#######################################################
#Edit this part to indicate which files and molecules to process
prefix='mglurMaster'
fnamelist=['20hz'+sys.argv[1], 'Theta'+sys.argv[1]]
#mollist=['2ag','Pkct', 'Dhpg', 'Dag', 'PkcCa','GaGTP']
mollist=['Pkct', '2ag', 'Ca']

#Headername can either be one of the files in the fname list or a distinct file
Headername='LongConcHeader.txt'

#A single mesh file means that all files in your list must use the same morphology
meshname='mesh.txt'

#indicate the name of the injection spines if you want to exclude them
fake = 'FAKE'
yes=1
no=0

spatialaverage=yes
#Spatial average only includes the following structure, and subdivides into bins:
dend="dend"
bins=10

prnvox=0
prnheader=0
###################################################

Avogadro=6.023e14 #to convert to nanoMoles

#FIRST PART = parse the header to determine identity/structure of voxels
f = open(Headername, 'r+')
data=f.readline()
if (prnheader==1):
    print data
else:
    print "header not printed"
f.close()

#Define some substrings used in reading header
space=' '
underscore='_'
dot='.'
vol='Vol'

#number of voxels in the file
maxvols=data.count(vol)
volnum=zeros(maxvols)
regionID=list()
structType=list()

#This parsing does not include reading the molecule species; thus it currently only works for single species files
oldvox=0
for i in range(maxvols):
    #find the 'Vol and second underscore which bracket the volume number
    newvox=data.find(space,oldvox+1)
    numstart=data.find(underscore,newvox )+1
    numend=data.find(underscore,numstart)
    volnum[i]=data[numstart:numend]
#find the region ID, such as dendrite, soma or spine number
    typestart=numend+1
    typendA=data.find(underscore,typestart)
    typendB=data.find(dot,typestart)
    
#if an underscore is found, this is a dendrite type of region, if a dot is found, this is structure in a spine
    if (typendA<typendB) or (typendB==-1):
        regionID.append(data[typestart:typendA])
    #if region, then the next string is either cyt or submemb
        structBegin=typendA+1
        structEnd=data.find(underscore,typendA+1)
        structType.append(data[structBegin:structEnd])
    else:
    #if not a region (if a dot found) then this is a structure, e.g. PSD, and the next string is the structure name
        structType.append(data[typestart:typendB])
        typestart=typendB+1
        typendA=data.find(underscore, typestart)
        regionID.append(data[typestart:typendA])
    
    #now update the starting point by making the present voxel the old voxel to search for the next space
    oldvox=newvox

print volnum, regionID, structType

#all voxels should be read in now with labels
#extract number of unique regions (e.g. dendrite, or sa1[0]), 
#and create list of subvolumes which contribute to that region

regionList=list()
regionVox=list()
regionCol=list()
regionStructList=list()
regionStructVox=list()
regionStructCol=list()

for i in range(maxvols):
    if (structType[i] != fake):
        unique=yes
        regionStruct=regionID[i]+structType[i]
        uniqueStruct=yes
#first construct list of region specific voxels
        j=0
        while ((unique==yes) and (j<len(regionList))):
            if regionID[i]==regionList[j]:
                regionVox[j].append(volnum[i])
                regionCol[j].append(i)
                unique=no
            #endif
            j+=1
        #endwhile
        if (unique==yes):
            regionList.append(regionID[i])
            regionVox.append([])
            regionCol.append([])
            regionVox[len(regionList)-1].append(volnum[i])
            regionCol[len(regionList)-1].append(i)
        #endif
       
#second construct list of region and structure specific voxels.
        j=0
        while ((uniqueStruct==yes) and (j<len(regionStructList))):
            if (regionStruct==regionStructList[j]):
                regionStructVox[j].append(volnum[i])
                regionStructCol[j].append(i)
                uniqueStruct=no
            #endif
            j+=1
        #endwhile
        if (uniqueStruct==yes):
            regionStructList.append(regionStruct)
            regionStructVox.append([])
            regionStructCol.append([])
            regionStructVox[len(regionStructList)-1].append(volnum[i])
            regionStructCol[len(regionStructList)-1].append(i)
        #endif

 #end for i
 
#next, read in the mesh file which has the volume for each voxel in column 13
volcol=13
#the x and y data can be used to create spatial averages over the dendrite
#sum of volumes within a region will be used for calculating mean concentration
meshdata=loadtxt(meshname,skiprows=1)
print '1st mesh file row:', meshdata[0,:]
volume=meshdata[:,volcol]
xloc=meshdata[:,1]
yloc=meshdata[:,2]

if (spatialaverage==yes):
    binmin=zeros(bins+1)
    binvoxels=[]
    bincolumns=[]
    xdim=max(xloc)-min(xloc)
    ydim=max(yloc)-min(yloc)

    #First, create the bins
    #xdim is larger:
    if (xdim >= ydim):
        bininc=xdim/bins
        minloc=min(xloc)
        maxloc=max(xloc)
        loc=xloc
        spaceheader='#time, x='
    #ydim is larger:
    else:
        bininc=ydim/bins
        minloc=min(yloc)
        maxloc=max(yloc)
        loc=yloc
        spaceheader='#time, y='
#now that assignmenst are made, determine bins the same for x or y direciton
    for j in range(bins):
        binmin[j]=minloc+j*bininc
        binvoxels.append([])
        bincolumns.append([])
    binmin[bins]=maxloc+bininc/bins
     
    #Now assign voxels to bins, either by x or y location
    print "binmin: ",binmin[:]
    print "vols=", maxvols, "regions=", len(regionID), "structures=", len(structType), "meshfile=", len(volume), "volnums=", len(volnum)
    for k in range(len(volnum)):
        if (regionID[k]==dend):
            j=0
            assigned=no
            while ((j < bins) and (assigned==no)):
                if ((loc[k]>=binmin[j]) and (loc[k] < binmin[j+1])):
                    binvoxels[j].append(volnum[k])
                    bincolumns[j].append(k)
                    assigned=yes
                j=j+1
            
    #now calculate volume for these spatial bins
    SpatialVol=zeros(bins)
    print "\nFOR spatial average: j, vol, binvoxels, bincolums:"
    for j in range(bins):
        for k in binvoxels[j]:
            SpatialVol[j]+=volume[k]
        if (prnvox==1):
            print j, SpatialVol[j], binvoxels[j], bincolumns[j]
        else:
            print "not printed"

    for j in range(bins):
        spaceheader=spaceheader+' '+str(format(binmin[j],'.1f'))+'-'+str(format(binmin[j]+bininc,'.1f'))
    print 'Space header:', spaceheader

#endif for spatial average

#This volume is in units of cubic microns, multiply by 1e-15 to convert to Liters
RegVol=zeros(len(regionList))
TotVol=0
for k in range(maxvols):
    TotVol+=volume[k]
print "TotVol:", TotVol

print "\nFOR region average: j, regionvox,Regioncol:"
for j in range(len(regionList)):
    for k in regionVox[j]:
        RegVol[j]+=volume[k] 
    if (prnvox==1):
        print j, regionVox[j],regionCol[j],RegVol[j]
    else:
        print "not printed"

RegStructVol=zeros(len(regionStructList))
print "\nFOR region-structure average: j, structvox,structcol:"
for j in range(len(regionStructList)):
    for k in regionStructVox[j]:
        RegStructVol[j]+=volume[k] 
    if (prnvox==1):
        print j, regionStructVox[j],regionStructCol[j], RegStructVol[j]
    else:
        print "not printed"

#Lastly, read in the data, this should be similar to AddMolecules.py
#Can do all molecules in a list without a batch file

header='#time'
for j in range(len(regionList)):
    header=header+' '+regionList[j]
for j in range(len(regionStructList)):
    header=header+' '+regionStructList[j]
header=header+' AvgTotal\n'
print 'header:', header

for filenum in range(len(fnamelist)):
    
    for mol in mollist:
	fullname=prefix+fnamelist[filenum]+'-'+mol+'-conc.txt'
        alldata=loadtxt(fullname,skiprows=1)
        print fullname

        time=alldata[:,0]
        data=alldata[:,1:alldata.shape[1]]
        del alldata
#the above eliminates the time column from the data, so that e.g., column 0 = voxel 0
        
##now, calculate various averages such as soma and dend, submemb vs cyt, taking advantage of the lists
#calculate Total volume of each region, and each region-structure
        
        RegionMeans=zeros((len(time),len(regionList)))
        for itime in range(len(time)):
            for j in range(len(regionList)):
                for k in regionCol[j]:
                    RegionMeans[itime,j]+=data[itime,k]
#sum the molecules of the voxels in the structure, divide by Avogadro and total volume (1e-15 converts from u^3 to L)
        
        for j in range(len(regionList)):
            RegionMeans[:,j]/=(RegVol[j]*Avogadro*1e-15)
        
#Repeat for regionStructures and overall mean
        RegionStructMeans=zeros((len(time),len(regionStructList)))
        OverallMean=zeros(len(time))
        
        for itime in range(len(time)):
            for j in range(len(regionStructList)):
                for k in regionStructCol[j]:
                    RegionStructMeans[itime,j]+=data[itime,k]
            for k in range(maxvols):
                OverallMean[itime]+=data[itime,k]
        
        for j in range(len(regionStructList)):
            RegionStructMeans[:,j]/=(RegStructVol[j]*Avogadro*1e-15)
        
        OverallMean[:]/=(TotVol*Avogadro*1e-15)
        
        SpatialMeans=zeros((len(time),bins))
        if (spatialaverage==yes):
            for itime in range(len(time)):
                for j in range(bins):
                    for k in bincolumns[j]:
                        SpatialMeans[itime,j]+=data[itime,k]
            for j in range(bins):
                print "j, vol=", j, SpatialVol[j]
                print SpatialMeans[1:10,j]
                if (SpatialVol[j] != 0):
                    SpatialMeans[:,j]/=(SpatialVol[j]*Avogadro*1e-15)
                print SpatialMeans[1:10,j]

#write average        
	outfname=fnamelist[filenum]+'-'+mol+'_avg.txt'
        print 'output file: ', outfname
	outdata=column_stack((time,RegionMeans,RegionStructMeans,OverallMean))
        f=open(outfname, 'w')
        f.write(header)
	savetxt(f, outdata, fmt='%.4f', delimiter=' ')
        f.close()

#write space
        if (spatialaverage==yes):
            outnamespace=fnamelist[filenum]+'-'+mol+'_space.txt'
            outdata=column_stack((time,SpatialMeans))
            f=open(outnamespace, 'w')
            f.write(header+'\n')
            savetxt(f, outdata, fmt='%.4f', delimiter=' ')
            f.close()
        #end if space
    #end for mol
#end for filename
